@@ -1,5 +1,6 @@
 import logging
 
+import sys
 from pathlib import Path
 from functools import wraps
 from datetime import datetime
@@ -7,10 +8,11 @@ import random
 
 import tkinter as tk
 from . import MODULES_VERBOSE
-from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform, ScrolledWidget
-from . import Button, Checkbox, Entry, Label, LabelStateful, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree, ListboxControl
+from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform, ScrolledWidget, FrameRadio
+from . import Button, Checkbox, Entry, Label, LabelStateful, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree, Spinbox, Radio
+from . import ListboxControl, CheckboxFrame
 from . import var, spec, fn
-from .model import CP, BindingGlobal, Timeout, TimeoutIdle, FileType, FileTypes
+from .model import CP, BindingGlobal, Timeout, TimeoutIdle, Interval, RateLimiter, FileType, FileTypes, Justification, WindowState
 
 
 logger = logging.getLogger(__name__)
@@ -20,12 +22,15 @@ class LuftBaloons(FrameUnlabelled):
     layout = 'xE'
 
     def setup_widgets(self, howmany=16 - 3):
+        assert howmany > 1
         widgets = {}
         for widx in range(howmany):
             idx = 'lb:%d' % widx
             widget = Checkbox(self, label='%02d' % widx)
             widget.trace(self.onClick_lb, idx=idx)
             widgets[idx] = widget
+        # Ignore the second checkbox
+        fn.state_ignore(widgets['lb:1'])
         return widgets
 
     def onClick_lb(self, vobj, etype, *, idx):
@@ -40,22 +45,36 @@ class ListFrame_Inner(FrameStateful):
     def __init__(self, *args, label, **kwargs):
         super().__init__(*args, label=label, labelInner=label, **kwargs)
 
-    def setup_widgets(self, labelInner):
-        self.lbl = Label(self, label=f'Label: {labelInner}')
-        self.e = Entry(self)  # label=f'Entry: {labelInner}'
+    def setup_widgets(self, labelInner, *, ljust=Justification.NoJustify):
+        self.lbl = Label(self, label=f'Label: {labelInner}', justify=ljust)
+        self.e = Entry(self, justify=ljust)  # label=f'Entry: {labelInner}'
+
+
+class ListFrame_Outer_Label(FrameUnlabelled):
+    layout = tk.HORIZONTAL
+
+    def setup_widgets(self):
+        self.cbL = Checkbox(self, label='').putIgnoreState()
+        self.lbl = Label(self, label='Child Widgets\nare Justified',
+                         anchor=CP.N, expand=True)
+        self.cbR = Checkbox(self, label='').putIgnoreState()
 
 
 class ListFrame_Outer(FrameStateful):
     label = 'Outer Frame'
-    layout = 'R2,1'
+    layout = 'R1,2,1'
 
     def setup_widgets(self, *, cbox1):
+        self.lbls = ListFrame_Outer_Label(self)
         self.left = ListFrame_Inner(self, label='Left',
-                                    cvariableDefault=True)
+                                    cvariableDefault=True,
+                                    ljust=Justification.Left)
         self.right = ListFrame_Inner(self, label='Right',
-                                     cvariableDefault=False)
-        self.bottom = ListFrame_Inner(self, label='Bottom',
-                                      cvariable=cbox1)
+                                     cvariableDefault=False,
+                                     ljust=Justification.Right)
+        self.bottom = ListFrame_Inner(self, label='Center',
+                                      cvariable=cbox1,
+                                      ljust=Justification.Center)
 
 
 class ListFrame_Lists(FramePaned):
@@ -66,18 +85,37 @@ class ListFrame_Lists(FramePaned):
                             height=6,
                             variable=vLst,
                             selectable=True)  # Selectable
-        self.lstRO = Listbox(self, label='Unselectable',
-                             maxHeight=3, expand=True,  # Varying Height, Expanded  # N/A on FramePaned
-                             variable=vLst,
-                             selectable=False,
-                             style=Listbox.Style(altbg=True),
-                             )  # Not Selectable
+        self.lstRO = ScrolledWidget(self, Listbox, label='Unselectable',
+                                    maxHeight=3, expand=True,  # Varying Height, Expanded  # N/A on FramePaned
+                                    variable=vLst,
+                                    selectable=False,
+                                    style=Listbox.Style(altbg=True),
+                                    )  # Not Selectable
+
+
+class ListFrame__Actions(FrameUnlabelled):
+    layout = tk.HORIZONTAL
+
+    def setup_widgets(self):
+        self.op1 = Button(self, label='Op1',
+                          styleID='Small')
+        self.op2 = Button(self, label='Op2',
+                          styleID='Small')
+
+    def setup_adefaults(self):
+        self.op1.onClick = self.genAction('OP1')
+        self.op2.onClick = self.genAction('OP2')
+
+    def genAction(self, label: str):
+        def genAction(event=None):
+            logger.debug('Action: %s', label)
+        return genAction
 
 
 class ListFrame(FrameLabelled):
     layout = 'R3,1,2,1'
 
-    def setup_widgets(self, *, cbox1):
+    def setup_widgets(self, *, cbox1) -> None:
         i_choices: spec.StaticMap[int] = spec.StaticMapValues(lambda i: 'I%d' % i, range(10), defaultValue=7)
 
         self.bFill = Button(self, label='Fill Me!')
@@ -93,9 +131,14 @@ class ListFrame(FrameLabelled):
         self.rstateful = ListFrame_Outer(self, labelAnchor=CP.N,
                                          cbox1=cbox1)
 
+        self._actions = ListFrame__Actions(self).putHelper()
+
     def setup_layout(self, layout):
         fn.configure(self, self.cLst, rows=True, weight=0)
 
+        self._actions.place(anchor=CP.NE.value, relx=1, rely=0,
+                            x=-2, y=-17 if sys.platform == 'win32' else -19)
+
     def setup_defaults(self):
         self.fill_lst()
         # Events
@@ -136,10 +179,16 @@ class UpstreamBool(FrameLabelled):
     isNoneable = False  # Don't skip this widget, even when its state is `None`
 
     def setup_widgets(self, what_bool):
-        self.bNoOp = Button(self, label='noop')
+        self.bOnFS = Button(self, label='Toggle FullScreen')
         self.u_bool = Checkbox(self, variable=what_bool, label='Upstream "bool"')
         self.bNoOp_Big = Button(self, label='No Operation')
 
+    def setup_adefaults(self):
+        self.bOnFS.onClick = self.onRootFS
+
+    def onRootFS(self, event=None):
+        self.wroot.rgstate = WindowState(fullscreen=not self.wroot.rgstate.fullscreen)
+
 
 class NB_Child_Simple(FrameUnlabelled):
     layout = 'Rx,1'
@@ -149,11 +198,11 @@ class NB_Child_Simple(FrameUnlabelled):
         w = {}
         for n in range(5):
             w[f'n{n}'] = Label(self, label=f'{n}: {label}')
-        w['e'] = LabelStateful(self)  # label=f'LS #{n}'
+        w['e'] = LabelStateful(self, labelPosition=CP.E)
         return w
 
     def setup_defaults(self):
-        self.widgets['e'].onClick = self.onClick_E
+        self.widgets['e'].binding('<Button-1>', self.onClick_E)
 
     def setup_adefaults(self):
         self.widgets['e'].wstate = 'Clickable LabelStateful'
@@ -163,9 +212,12 @@ class NB_Child_Simple(FrameUnlabelled):
         state = w.wstate
         if not state.endswith(' T'):
             state += ' T'
+            imgname = 'warning-s16'
         else:
             state = state[:-2]
+            imgname = 'info-s16'
         w.wstate = state
+        w['image'] = self.wimage(imgname)
 
 
 class NB_Child_Complex(NotebookUniform):
@@ -183,8 +235,10 @@ class NB_Child_Timeout(FrameLabelled):
         super().__init__(*args, **kwargs)
 
     def setup_widgets(self):
-        self.cScheduled = Checkbox(self, label='Scheduled?', readonly=True)
-        self.cTriggered = Checkbox(self, label='Triggered?', readonly=True)
+        self.cScheduled = Checkbox(self, label='Scheduled?', readonly=True,
+                                   styleID='ReadonlyEmphasis')
+        self.cTriggered = Checkbox(self, label='Triggered?', readonly=True,
+                                   styleID='ReadonlyEmphasis')
         self.bToggle = Button(self, label='Toggle\nasync')
 
         self.bToggle.onClick = self.onToggle
@@ -331,6 +385,90 @@ class NB_Child_Timeouts(FrameUnlabelled):
         self.timeout_idle_c = NB_Child_TimeoutIdle_Chain(self)
 
 
+class NB_Child_Interval(FrameUnlabelled):
+    layout = 'R3,3'
+
+    def __init__(self, *args, **kwargs):
+        self.interval = Interval(self, self.onInterval, 1000, immediate=False)
+        super().__init__(*args, **kwargs)
+
+    def setup_widgets(self):
+        self.txt_lbl = Label(self, label='Count (1s)')
+        self.txt = Entry(self, justify=CP.center, readonly=True)
+        self.stateScheduled = Checkbox(self, label='Scheduled?', readonly=True)
+        self.state_on = Button(self, label='ON')
+        self.state_off = Button(self, label='OFF')
+        self.state_offforce = Button(self, label='OFF (Force)')
+
+    def setup_defaults(self):
+        self.txt.wstate = 'Elapsed Seconds'
+
+    def setup_adefaults(self):
+        self.state_on.onClick = self.onIntervalOn
+        self.state_off.onClick = self.onIntervalOff
+        self.state_offforce.onClick = self.onIntervalOffForce
+
+    def onIntervalOn(self, event=None):
+        if self.interval.scheduled:
+            logger.debug('Already Scheduled')
+        else:
+            self.txt.wstate = str(0)
+            self.interval.schedule()
+        self.stateScheduled.wstate = self.interval.scheduled
+        self.state_off.focus()
+
+    def onIntervalOff(self, event=None):
+        self.interval.unschedule()
+        self.state_on.focus()
+        self.stateScheduled.wstate = self.interval.scheduled
+
+    def onIntervalOffForce(self, event=None):
+        was_scheduled = self.interval.scheduled
+        self.interval.unschedule(force=True)
+        if was_scheduled:
+            self.txt.wstate = f'Force Stop at {self.txt.wstate}'
+        self.stateScheduled.wstate = self.interval.scheduled
+        self.state_on.focus()
+
+    def onInterval(self):
+        new_state = str(int(self.txt.wstate) + 1)
+        if self.interval.scheduled:
+            self.txt.wstate = new_state
+        else:
+            self.txt.wstate = f'Stop at {new_state}'
+
+
+class NB_Child_RateLimiter(FrameUnlabelled):
+    layout = 'R2,1,2'
+
+    def __init__(self, *args, **kwargs):
+        self.rl = RateLimiter(self, self.onRL, 1000)
+        super().__init__(*args, **kwargs)
+
+    def setup_widgets(self):
+        self.txt_lbl = Label(self, label='Now')
+        self.txt = Entry(self, justify=CP.center, readonly=True, width=30)
+        self.hit_lbl = Label(self, label='Hit me Hard!\nCount will only change once per second.\nThe timings are not perfect.')
+        self.buttonHit = Button(self, label='HIT')
+        self.stateRL = Checkbox(self, label='Rate Limited?', readonly=True)
+
+    def setup_defaults(self):
+        self.txt.wstate = str(0)
+
+    def setup_adefaults(self):
+        self.buttonHit.onClick = self.onHit
+
+    def onHit(self, event=None):
+        self.stateRL.wstate = not self.rl.hit()
+
+    def onRL(self):
+        self.txt.wstate = f"{datetime.now().isoformat(' ', timespec='microseconds')} μs"
+        # self.txt.wstate = str(int(self.txt.wstate) + 1)
+        if self.stateRL.wstate:
+            logger.debug('Clear RateLimit marker')
+            self.stateRL.wstate = False
+
+
 class NB_Child_Dialog(FrameUnlabelled):
     layout = 'Rx,1'
     wstate_single = 'txt'
@@ -405,14 +543,14 @@ class NB_Child_Scrollbars(FrameUnlabelled):
 
     def onShowAll(self, event=None):
         # sbL: Auto, Set True
-        self.sbL.set_scroll_state(True, True)
+        self.sbL.wproxy.set_scroll_state(True, True)
         # sbC: Manual, Set True (no-op)
-        self.sbC.set_scroll_state(True, True)
+        self.sbC.wproxy.set_scroll_state(True, True)
         # sbR: Manual, Set Reversed
-        self.sbR.set_scroll_state(*(not b for b in self.sbR.get_scroll_state()))
+        self.sbR.wproxy.set_scroll_state(*(not b for b in self.sbR.wproxy.get_scroll_state()))
 
 
-class NB_Child_ListboxSet(FrameUnlabelled):
+class NB_Complex_ListboxSet(FrameUnlabelled):
     layout = tk.HORIZONTAL
 
     def setup_widgets(self):
@@ -429,26 +567,93 @@ class NB_Child_ListboxSet(FrameUnlabelled):
         self.left.onAddAll()
 
 
+class NB_Complex_CheckboxList(CheckboxFrame):
+    def __init__(self, *args, **kwargs):
+        kwargs.update({
+            'stateCheckboxes': {
+                '%d' % n: 'Check #%d' % n
+                for n in range(0, 15, 3)
+            },
+            'layoutCheckboxes': '2x3',
+        })
+        super().__init__(*args, **kwargs)
+
+
+class NB_Complex_CheckboxNested(CheckboxFrame):
+    def __init__(self, *args, **kwargs):
+        kwargs.update({
+            'layout': 'H1,x',
+            'stateCheckboxes': {
+                **{
+                    'N1(%d)' % nn: {
+                        'N2%d_%d' % (nn, nnn): 'N2(%d|%d)' % (nn, nnn)
+                        for nnn in range(2)
+                    }
+                    for nn in range(3)
+                },
+                **{
+                    'NN1': {
+                        'NN2_0': 'NN2(0)',
+                        **{
+                            'NN2(%d)' % N: {
+                                'NN3%d_%d' % (N, NN): 'NN3(%d|%d)' % (N, NN)
+                                for NN in range(3)
+                            }
+                            for N in range(1, 2 + 1)
+                        },
+                    },
+                },
+            },
+            'layoutCheckboxes': [tk.HORIZONTAL, tk.VERTICAL, tk.VERTICAL],
+            'traceFn': self.onCheckboxClick,
+        })
+        super().__init__(*args, **kwargs)
+
+    def onCheckboxClick(self, what):
+        def onCheckboxClick(var, etype):
+            logger.debug('Trace: %s [%s]', what, var.get())
+        return onCheckboxClick
+
+
+class NB_Complex(Notebook):
+    def setup_tabs(self):
+        return {
+            'lc': Notebook.Tab('ListboxControl', NB_Complex_ListboxSet(self)),
+            'cl': Notebook.Tab('CheckboxFrame', NB_Complex_CheckboxList(self)),
+            'cn': Notebook.Tab('CheckboxFrame\nNested', NB_Complex_CheckboxNested(self)),
+        }
+
+    def setup_adefaults(self):
+        self.wselect('cn')
+
+
 class NB(Notebook):
     def setup_tabs(self):
-        logger.debug('» %r', self)
-        # TODO: Test images, etc.
-
         return {
             'sb': Notebook.Tab('Scrollbars', NB_Child_Scrollbars(self)),
             'tt': Notebook.Tab('Timeouts', NB_Child_Timeouts(self),
                                image=self.wimage('info-s16'), labelPosition=CP.E),  # Default labelPosition
+            'ti': Notebook.Tab('Interval', NB_Child_Interval(self)),
+            'trl': Notebook.Tab('RateLimiter', NB_Child_RateLimiter(self)),
             'td': Notebook.Tab('Dialogues', NB_Child_Dialog(self),
                                image=self.wimage('info-msgbox-s16'), labelPosition=True),  # Only image
-            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self),
+            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self, styleID='TabCenter'),
                                image=self.wimage('info-s16'), labelPosition=CP.W),  # "Reverse" labelPosition
             't1': Notebook.Tab('Tab 1', NB_Child_Simple(self, label='Tab 1')),
             't2': Notebook.Tab('Tab 2', NB_Child_Simple(self, label='Tab 2')),
-            'c:lc': Notebook.Tab('Complex ListboxControl', NB_Child_ListboxSet(self)),
+            'c': Notebook.Tab('Complex', NB_Complex(self, styleID='TabV_W')),  # Vertical Tabs (not fully baked)
         }
 
     def setup_adefaults(self):
-        self.wselect('c:lc')
+        self.wselect('c')
+
+
+class TextWidget(EntryMultiline):
+    styleSheet = {
+        '.emph': {
+            'overstrike': True,
+        }
+    }
 
 
 class TextEditor(FrameStateful):
@@ -460,19 +665,19 @@ class TextEditor(FrameStateful):
         self.bTxtClean = Button(self, label='Clean TXT')
         self.bTxtReset = Button(self, label='Reset TXT')
         self.bTxtSet = Button(self, label='Set TXT')
-        self.txt = ScrolledWidget(self, EntryMultiline,
+        self.txt = ScrolledWidget(self, TextWidget,
                                   setgrid=False)
-        # Setup events
-        self.bTxtClean.onClick = self.txt.style_reset
-        self.bTxtReset.onClick = self.txt_reset
-        self.bTxtSet.onClick = self.txt_set
-        self.txt.onClickTag = self.txt_clicked
 
     def setup_layout(self, layout):
         self.txt.grid(sticky=tk.NS)
 
     def setup_defaults(self):
         self.rowconfigure(0, weight=0)  # Buttons
+        # Setup events
+        self.bTxtClean.onClick = self.txt.style_reset
+        self.bTxtReset.onClick = self.txt_reset
+        self.bTxtSet.onClick = self.txt_set
+        self.txt.onClickTag = self.txt_clicked
 
     def setup_adefaults(self):
         self.txt_set(cnt=20)
@@ -483,9 +688,9 @@ class TextEditor(FrameStateful):
     def txt_set(self, event=None, *, cnt=None):
         texts = []
         if cnt is None:
-            cnt = 20 + random.choice(range(-10, 5))
+            cnt = random.choice(range(10, 25))
         for r in range(cnt):
-            texts.append(f'<b>Line</b> <i>{"%02d" % r}</i>/{cnt} ')
+            texts.append(f'<b class="emph">Line</b> <i>{"%02d" % r}</i>/{cnt} ')
             if r % 2 == 0:
                 _txt = 'EVEN'
             else:
@@ -505,13 +710,192 @@ class TreeExample(Tree):
         logger.debug('Clicked on header "%s"', header.name)
 
 
-class ComboboxCB(Combobox):
-    pass
+class NB_Center_Spinbox(FrameUnlabelled):
+    layout = 'x2E'
+
+    def setup_widgets(self):
+        lim = spec.Limit(1, 5, fn=int, imax=False)
+
+        self.label_spinN = Label(self, label=f'{lim} :: Normal')
+        self.spinN = Spinbox(self, values=lim)
+
+        self.label_spinL = Label(self, label=f'{lim} :: Wrap Values')
+        self.spinL = Spinbox(self, values=lim, wrap=True)
+
+        self.label_spinE = Label(self, label=f'{lim} :: Editable')
+        self.spinE = Spinbox(self, values=lim, readonly=False)
+
+        lim_grow = spec.Limit(None, None, fn=int, imax=False)
+
+        self.label_spinINF = Label(self, label=f'{lim_grow} :: Unlimited')
+        self.spinINF = Spinbox(self, values=lim_grow)
+
+
+class NB_Center_Combobox(FrameUnlabelled):
+    layout = 'x2E'
+    wstate_single = 'choice'
+
+    def setup_widgets(self):
+        choices = spec.StaticList(('Choice %d' % n for n in range(5)), defaultIndex=3)
+
+        vcb = self.var(var.String, name='choice')
+
+        self.label_choiceRO = Label(self, label='CB',
+                                    image=self.wimage('error-s16'),
+                                    labelPosition=CP.S)
+        self.choiceRO = Combobox(self, variable=vcb, values=choices)  # label='CB'
+
+        self.label_choiceRW = Label(self, label='CB [RW]',
+                                    image=self.wimage('warning-s16'),
+                                    labelPosition=CP.E)
+        self.choiceRW = Combobox(self, variable=vcb, values=choices,
+                                 readonly=False)  # label='CB [RW]'
+
+        self.choice_reset = Button(self, label='CB: Set Last', image=self.wimage('info-s16'))
+        self.choice_reset.onClick = self.choiceRO.eSet(choices[-1])
+
+
+class NB_Center_Radio(FrameUnlabelled):
+    layout = 'x3E'
+    wstate_single = 'radio'
+
+    def setup_widgets(self):
+        choices = spec.StaticList(('%d' % n for n in range(1, 9 + 1)), defaultIndex=9 // 2)
+        rvar = self.varSpecced(var.SpeccedString, name='radio', spec=choices)
+
+        widgets = {}
+        for c in choices.all():
+            widgets[c] = Radio(self, label=f'§ {c} §',
+                               variable=rvar, value=c)
+        widgets[':button'] = Button(self, label='Check State')
+        widgets[':button'].onClick = self.onCheckState
+        return widgets
+
+    def onCheckState(self):
+        logger.debug('State: %s', self.wstate)
+        for wname, w in self.widgets.items():
+            if not wname.startswith(':'):
+                logger.debug('- %s: %s', w['text'], w.isSelected())
+
+
+class NB_Center_RadioFrames_Inner(FrameRadio):
+    wstate_single = 'inner'
+
+    def setup_widgets(self, n):
+        self.lbl = Label(self, label=f'Inner\nRadio Frame\n{n}')
+        self.inner = ListFrame_Inner(self, label=n)
+
+
+class NB_Center_RadioFrames_FInner(FrameStateful):
+    layout = tk.HORIZONTAL
+
+    def setup_widgets(self, rvar):
+        widgets = {}
+        for c in rvar.spec.all():
+            widgets[f'f:{c}'] = NB_Center_RadioFrames_Inner(self, label=f'Frame {c}',
+                                                            rvariable=rvar, rvalue=c,
+                                                            n=c)
+        return widgets
+
+
+class NB_Center_RadioFrames(FrameUnlabelled):
+    layout = 'x2E'
+
+    def setup_widgets(self):
+        choices = spec.StaticList(iter(('LeftSide', 'RightSide')), defaultIndex=0)
+        rvar = self.varSpecced(var.SpeccedString, name='frame', spec=choices)
+
+        widgets = {}
+        for c in choices.all():
+            widgets[f'r:{c}'] = Radio(self, label=f'Radio {c}',
+                                      variable=rvar, value=c)
+        widgets['finner'] = NB_Center_RadioFrames_FInner(self, label='Stateful', rvar=rvar)
+        return widgets
+
+    def setup_defaults(self):
+        self.rowconfigure(0, weight=0)  # Radio Buttons
+
+
+class NB_Center_LabelAlignSL(FrameUnlabelled):
+    layout = '3x5E'
+
+    def setup_widgets(self):
+        widgets = {}
+        justifies = [None, *Justification]
+        for justify in justifies:
+            wkey = [
+                'xpand',
+                'justify' if justify is None else justify.name,
+            ]
+            widgets['_'.join(wkey)] = Label(self, label=' '.join(wkey), styleID='ShowBG|WarnFG',
+                                            image=self.wimage('warning-s16'), labelPosition=CP.S,
+                                            justify=justify)
+        for expand in [True, False]:
+            for justify in justifies:
+                wkey = [
+                    'X' if expand else 'x',
+                    'justify' if justify is None else justify.name,
+                ]
+                widgets['_'.join(wkey)] = Label(self, label=' '.join(wkey), styleID='ShowBG',
+                                                image=self.wimage('info-s16'), labelPosition=CP.S,
+                                                justify=justify, expand=expand)
+        return widgets
+
+    def setup_layout(self, layout):
+        for w in self.widgets.values():
+            w.grid(padx=2, pady=2)
+
+
+class NB_Center_LabelAlignML(FrameUnlabelled):
+    layout = '4x6S'
+
+    def setup_widgets(self):
+        widgets = {}
+        for anchor in [None, CP.N, CP.S, CP.E, CP.W, CP.center]:
+            for justify in [None, Justification.Left, Justification.Center, Justification.Right]:
+                wkey = [
+                    'anchor' if anchor is None else anchor.name,
+                    'justify' if justify is None else justify.name,
+                ]
+                widgets['_'.join(wkey)] = Label(self, label='\n'.join(wkey), styleID='ShowBG',
+                                                anchor=anchor, justify=justify)
+        return widgets
+
+    def setup_layout(self, layout):
+        for w in self.widgets.values():
+            w.grid(padx=2, pady=2)
+
+
+class NB_Center(Notebook):
+    def setup_tabs(self):
+        return {
+            'cb': Notebook.Tab('Combobox', NB_Center_Combobox(self)),
+            'spin': Notebook.Tab('Spinbox', NB_Center_Spinbox(self)),
+            'label_align:sl': Notebook.Tab('Label Align:SL', NB_Center_LabelAlignSL(self)),
+            'label_align:ml': Notebook.Tab('Label Align:ML', NB_Center_LabelAlignML(self)),
+            'radio': Notebook.Tab('Radio', NB_Center_Radio(self)),
+            'frame:radio': Notebook.Tab('RadioFrames', NB_Center_RadioFrames(self)),
+        }
+
+    def setup_adefaults(self):
+        self.wselect('label_align:sl')
 
 
 class RW(RootWindow):
+    styleIDs = {
+        'TabCenter.TNotebook': {'tabposition': tk.N},
+        'TabV_W.TNotebook': {'tabposition': tk.W + tk.N},
+        'TabV_W.TNotebook.Tab': {'font': 'TkFixedFont'},
+        'ReadonlyEmphasis.TCheckbutton': {},
+        'ShowBG.TLabel': {'background': 'lightblue'},
+        'WarnFG.TLabel': {'foreground': 'darkred'},
+        'Small.TButton': {
+            'font': 'TkSmallCaptionFont',
+            'padding': '-3' if sys.platform == 'win32' else '-1',
+        },
+    }
+
     def setup_widgets(self):
-        choices = spec.StaticList(('Choice %d' % n for n in range(5)), defaultIndex=3)
         vc = self.var(var.Boolean, name='bool', value=True)
 
         self.b1 = Button(self, label='B1')
@@ -522,23 +906,22 @@ class RW(RootWindow):
         self.c2rw = Checkbox(self, label='RW "bool"', readonly=False, variable=vc)
         self.e1 = Entry(self, expand=True)  # label='Entry1'
         self.lbs = LuftBaloons(self)
+        self.nb_center = NB_Center(self)
         self.ubox = UpstreamBool(self, label='Upstream', labelAnchor=CP.S,
                                  what_bool=vc)
-        self.choice = ComboboxCB(self, values=choices)  # label='CB'
-        self.choice_reset = Button(self, label='CB: Set Last')
+        self.nb_center_label = Label(self, label='↰ ↑ ↱')  # Placeholder to keep the layout
         self.lf = ListFrame(self, label='List Box', cbox1=self.c1.variable)
 
         self.txt = TextEditor(self)
         self.nb = NB(self)
         self.arbre = TreeExample(self, label=Tree.Column('Label', image=self.wimage('warning-s16')), columns={
             'number': Tree.Column('Number', image=self.wimage('info-s16'), nameAnchor=CP.W, cellAnchor=CP.E),
-        }, columns_stretch=None)
+        }, columns_stretch=None, columns_autosize=None)  # For Test: columns_stretch=['number'])
 
         # Setup events
         self.b1.onClick = self.c1.toggle
         self.bE1.onClick = self.ask_contents
         self.b2.onClick = self.debug
-        self.choice_reset.onClick = self.choice.eSet(choices[-1])
 
     def setup_defaults(self):
         BindingGlobal(self, '<F4>', lambda e: self.debug(),
@@ -565,6 +948,8 @@ class RW(RootWindow):
         logger.debug('Global Bindings:')
         for bname, B in self._bindings_global.items():
             logger.debug('- %s: %s%s', bname, '' if B else '[Disabled] ', B.description)
+        logger.debug('Setup traces')
+        self.c2ro.trace(self.onTraceBool, trace_initial=True)
 
     def ask_contents(self):
         string = tk.simpledialog.askstring('Set Contents', f'Set the "{self.e1.label}" contents')
@@ -589,30 +974,40 @@ class RW(RootWindow):
         new = self.wstate
         for b in ('0', '12'):
             new['lbs'][f'lb:{b}'] = not new['lbs'][f'lb:{b}']
+        assert 'lb:1' not in new['lbs']  # Ignore the second checkbox
         self.wstate = new
         logging.info('=> GUI States')
         logging.info('   GUI State @ %s[%r]', self, self)
         for line in pformat(self.gstate).splitlines():
-            pass  # logging.info('  %s', line)
+            pass  # logging.debug('| %s', line)
         logging.info('   GUI State @ c2ro')
         for line in pformat(self.c2ro.gstate).splitlines():
-            pass  # logging.info('  %s', line)
+            pass  # logging.debug('| %s', line)
         logging.info('   GUI State @ c2rw')
         for line in pformat(self.c2rw.gstate).splitlines():
-            pass  # logging.info('  %s', line)
-        logging.info('   GUI State @ choice')
-        for line in pformat(self.choice.gstate).splitlines():
-            pass  # logging.info('  %s', line)
+            pass  # logging.debug('| %s', line)
+        logging.info('   GUI State @ nb_center')
+        for line in pformat(self.nb_center.gstate).splitlines():
+            pass  # logging.debug('| %s', line)
         logging.info('   GUI State @ e1')
         for line in pformat(self.e1.gstate).splitlines():
-            pass  # logging.info('  %s', line)
+            pass  # logging.debug('| %s', line)
         logging.info('   GUI State @ txt.txt')
         for line in pformat(self.txt.txt.gstate).splitlines():
-            pass  # logging.info('  %s', line)
+            pass  # logging.debug('| %s', line)
+        logger.info('=> Window State')
+        logger.debug('   %r', self.rgstate)
         # self.set_gui_state(enabled=True, valid=True)  # Invalid Target
-        logging.info('=> NB')
-        for tn, ti in self.nb.wtabs.items():
-            logging.info(f'   | {tn}: {ti}')
+        # logging.info('=> NB')
+        # for tn, ti in self.nb.wtabs.items():
+        #     logging.debug(f'| {tn}: {ti}')
+        logging.info('=> GridSize')
+        logging.debug('  %s', self.txt.gsize)
+
+    def onTraceBool(self, var, etype):
+        bool_state = var.get()
+        bool_when = 'Initial' if etype is None else 'Trigger'
+        logger.debug('Variable "%s" @ %s: %s', fn.vname(var), bool_when, bool_state)
 
 
 def entrypoint():
