@@ -10,13 +10,15 @@ from functools import cached_property, partial
 
 from . import fn
 from . import model
-from . import var
+from . import var as tkmilan_var  # Possible name alias
 from . import autolayout
+from . import spec
 
 import tkinter as tk
 
+ProxyWidgetT = typing.TypeVar('ProxyWidgetT', bound='MixinWidget')
 if typing.TYPE_CHECKING:
-    from . import RootWindow, ScrolledWidget
+    from . import RootWindow
 
 logger = logging.getLogger(__name__)
 
@@ -41,9 +43,9 @@ class ContainerState:
     '''
     swidgets: 'typing.Mapping[str, SingleWidget]'
     cwidgets: 'typing.Mapping[str, ContainerWidget]'
-    variables: 'typing.Mapping[str, var.Variable]'
-    wvariables: 'typing.Mapping[str, var.Variable]'
-    vwidgets: typing.Mapping[str, typing.Iterable[str]]
+    variables: 'typing.Mapping[str, tkmilan_var.Variable]'
+    wvariables: 'typing.Mapping[str, tkmilan_var.Variable]'
+    vwidgets: typing.Mapping[str, typing.Sequence[str]]
     vid_upstream: typing.Set[str]
 
 
@@ -52,7 +54,7 @@ class MixinState:
 
     wstate_static: bool = True
     '''
-    Define if the `setup_state` cache is static. or a callable for dynamic
+    Define if the `MixinState.setup_state` cache is static. or a callable for dynamic
     calculations.
 
     See `stateSetup`.
@@ -78,7 +80,7 @@ class MixinState:
 
         This can have a static object, or a dynamic `callable`.
 
-        See `wstate_static`, `stateSetup`.
+        See `MixinState.wstate_static`, `stateSetup`.
         '''
         raise NotImplementedError
 
@@ -89,7 +91,7 @@ class MixinState:
         This takes into account the `wstate_static` flag, producing a static object
         or a callable.
 
-        This should always be used, `setup_state` is only a definition.
+        This should always be used, `MixinState.setup_state` is only a definition.
         '''
         assert self.isNoneable is not None, f'{self} needs `isNoneable` choice'
         if self.wstate_static is True:
@@ -145,6 +147,13 @@ class MixinStateSingle(MixinState):
 class MixinStateContainer(MixinState):
     '''Mixin class for container widgets.
 
+    To ignore a container state, define this on the subclass:
+
+    .. code:: python
+
+        def setup_state(self, **kwargs):
+            return {}
+
     Note:
         When subclassing this, define `MixinState.setup_state` to return a
         dictionary mapping subwidget identifier to `WidgetDynamicState`.
@@ -170,17 +179,20 @@ class MixinStateContainer(MixinState):
                 pass  # Skip
             else:
                 state[identifier] = result
-        if self.wstate_single is not None:
-            assert len(state) == 1, f'{self}: Invalid "wstate_single" marking, {len(state)} elements'
-            return state[self.wstate_single]
-        else:
+        if self.wstate_single is None:
+            # - Multiple WState
             if len(state) == 0:
                 return None
             else:
                 if __debug__:
                     if len(state) == 1:
+                        # TODO: Use `warnings.warn`, but this is used from an `after` function, the trace is lost.
                         logger.warning('%s: This widget can be marked "wstate_single": `%s`', self, list(state)[0])
                 return state
+        else:
+            # - Single WState (wstate_single)
+            assert len(state) == 1, f'{self}: Invalid "wstate_single" marking, {len(state)} elements'
+            return state[self.wstate_single]
 
     def state_set(self, state, substate, **kwargs) -> None:
         # # Debug container state flow
@@ -193,9 +205,19 @@ class MixinStateContainer(MixinState):
         #                  '' if self.wstate_single is None else f' [{self.wstate_single}]',
         #                  )
         for identifier, wds in self.stateSetup(**kwargs).items():
-            skip = (wds.noneable and state is None) or \
-                ((substate or wds.noneable) and identifier not in state) or \
-                (self.wstate_single is not None and self.wstate_single != identifier)
+            # Skip State Setup:
+            # - "noneable" and the state is None
+            if wds.noneable and state is None:
+                skip = True
+            else:
+                if self.wstate_single is None:
+                    # - Multiple WState
+                    #   - Check for "noneable"
+                    #   - Check for (substate=True)
+                    skip = (substate or wds.noneable) and identifier not in state
+                else:
+                    # - Single WState (wstate_single)
+                    skip = self.wstate_single != identifier
             if skip:
                 # if __debug__:
                 #     logger.debug('%s|> Skip "%s"', ' ' * len(self_names), identifier)
@@ -229,20 +251,36 @@ class MixinWidget:
     gkeys: typing.FrozenSet[str]
     '''The supported `GuiState` keys.'''
     isHelper: bool
-    '''Marker that indicates the widgets is not part of the automatic state.'''
+    '''Marker that indicates the widget is not part of the automatic state.'''
     ignoreContainerState: bool = False
     '''Ignore this widget's state when included on a container.'''
+    styleID: typing.Optional[str] = None
+    '''StyleID for this widget. See `RootWindow.styleIDs`.'''
+    wproxy: 'typing.Optional[MixinWidget]' = None
+    '''Link to the corresponding proxy widget, if exists.
+
+    See Also:
+        The base `ProxyWidget` class.
+    '''
+    proxee: 'typing.Optional[MixinWidget]' = None
+    '''Link to the corresponding proxied widget, if exists.
+
+    See Also:
+        The base `ProxyWidget` class.
+    '''
     _bindings: typing.MutableMapping[str, model.Binding]
     '''Store all widget `Binding` objects, keyed by name (see `binding`).'''
     _tidles: typing.MutableMapping[str, model.TimeoutIdle]
     '''Store some widget `TimeoutIdle` objects, keyed by name (see `tidle`).'''
 
-    def __init__(self, *, gkeys: typing.Iterable[str] = None):
+    def __init__(self, *,
+                 gkeys: typing.Optional[typing.Iterable[str]] = None,
+                 ):
         self.isHelper: bool = getattr(self, 'isHelper', False)
         self._bindings = {}
         self._tidles = {}
         gk = set(model.GUI_STATES_common)
-        if gkeys:
+        if gkeys is not None:
             gk.update(gkeys)
         self.gkeys = frozenset(gk)
 
@@ -279,7 +317,7 @@ class MixinWidget:
         else:
             return self.wparent.wroot_search()
 
-    def binding(self, sequence: str, *args, key: str = None, immediate: bool = True, **kwargs) -> model.Binding:
+    def binding(self, sequence: str, *args, key: typing.Optional[str] = None, immediate: bool = True, **kwargs) -> model.Binding:
         '''Create a `model.Binding` for this widget.
 
         Stores all widget bindings on a per-instance dictionary, for later
@@ -303,7 +341,7 @@ class MixinWidget:
         self._bindings[name] = model.Binding(self, sequence, *args, immediate=immediate, **kwargs)
         return self._bindings[name]
 
-    def tidle(self, action: typing.Callable, *args, key: str = None, **kwargs) -> model.TimeoutIdle:
+    def tidle(self, action: typing.Callable, *args, key: typing.Optional[str] = None, **kwargs) -> model.TimeoutIdle:
         '''Create a `model.TimeoutIdle` for this widget.
 
         Stores all idle timeouts created using this function on a per-instance
@@ -411,7 +449,7 @@ class MixinWidget:
         #     logger.debug('State > %r', rval)
         return rval
 
-    def set_gui_state(self, state: typing.Optional[model.GuiState] = None, **kwargs) -> None:
+    def set_gui_state(self, state: typing.Optional[model.GuiState] = None, **kwargs) -> model.GuiState:
         assert isinstance(self, tk.ttk.Widget), f'{self} is not a valid tkinter.ttk.Widget'
         if state is None:
             state = model.GuiState(**kwargs)
@@ -429,36 +467,83 @@ class MixinWidget:
                 # logger.debug('  [%s %s] » %s', sval, itk.invert, tkstr)
                 states.append(tkstr)
         self.state(states)
+        assert state is not None
+        return state
 
     # Wrapper functions for the property
     def gstate_get(self):
         return self.get_gui_state()
 
     def gstate_set(self, state: model.GuiState):
-        return self.set_gui_state(state)
+        # Don't store the return object
+        self.set_gui_state(state)
 
     # TODO: This can be even better
     # Support `widget.gstate.enabled = NEW_VALUE`
     # Not a property, but a class that changes `self`
     gstate = property(gstate_get, gstate_set, doc='GUI State')
 
+    # TODO: On Python 3.11:: -> typing.Self
+    def putHelper(self, value: bool = True) -> 'MixinWidget':
+        '''Set the `isHelper` marker on itself.
+
+        This is designed to be used inside the `setup_widgets
+        <ContainerWidget.setup_widgets>` function, like this:
+
+        .. code:: python
+
+            def setup_widgets(self, ...):
+                self.w1 = Widget(self, ...).putHelper()
+
+        This is usually called "method chaining", or "fluent interface".
+        '''
+        self.isHelper = value
+        return self
+
+    # TODO: On Python 3.11:: -> typing.Self
+    def putIgnoreState(self, value: bool = True) -> 'MixinWidget':
+        '''Set the `ignoreContainerState` marker on itself.
+
+        This is designed to be used inside the `setup_widgets
+        <ContainerWidget.setup_widgets>` function, like this:
+
+        .. code:: python
+
+            def setup_widgets(self, ...):
+                self.w1 = Widget(self, ...).putIgnoreState()
+
+        This is usually called "method chaining", or "fluent interface".
+        '''
+        # TODO: Move this "method chaining" to a common document.
+        self.ignoreContainerState
+        return self
+
 
 class MixinTraces:
-    def init_traces(self):
-        self._traces: typing.MutableMapping[str, str] = {}
-        assert self.variable is not None, f'{self}: Widget untraceable'
+    '''Mixin class for variable traces.
 
-    def trace(self, function: typing.Callable, *, trace_mode: model.TraceModeT = 'write', trace_name: typing.Optional[str] = None, **kwargs: typing.Any) -> str:
+    Supported for `SingleWidget` only.
+    '''
+    def init_traces(self) -> None:
+        self._traces: typing.MutableMapping[str, str] = {}
         assert isinstance(self, SingleWidget), f'{self.__class__.__name__}: Unsupported tracing for this Widget'
         assert self.variable is not None, f'{self}: Widget untraceable'
-        function_name = self.variable.trace_add(
-            trace_mode,
-            fn.generate_trace(
-                self.variable,
-                function,
-                **kwargs,
-            ),
-        )
+
+    def trace(self, function: typing.Callable, *, trace_name: typing.Optional[str] = None, **kwargs: typing.Any) -> str:
+        '''Trace the variable associated to the current `SingleWidget`.
+
+        The underlying function is `tkmilan.var.trace`, check it for more
+        detailed documentation.
+
+        Args:
+            function: The callback function.
+            trace_name: A name for the trace reference. Must be unique for the
+                widget. Optional, uses an automatic name otherwise.
+            kwargs: Passed to the `tkmilan.var.trace` function.
+        '''
+        assert isinstance(self, SingleWidget), f'{self.__class__.__name__}: Unsupported tracing for this Widget'
+        assert self.variable is not None, f'{self}: Widget untraceable'
+        function_name = tkmilan_var.trace(self.variable, function, **kwargs)
         key = trace_name or function_name
         assert key not in self._traces, f'{self}: Repeated trace name: {key}'
         self._traces[key] = function_name
@@ -470,21 +555,27 @@ class MixinTraces:
 
 class SingleWidget(MixinWidget, MixinStateSingle, MixinTraces):
     '''Parent class of all single widgets.'''
-    variable: typing.Optional[var.Variable] = None
-    state_type: typing.Optional[typing.Type[var.Variable]] = None
-    scrollFrame: 'typing.Optional[ScrolledWidget]' = None
-    '''Link to the corresponding `ScrolledWidget`, if exists'''
+    variable: typing.Optional[tkmilan_var.Variable] = None
+    state_type: typing.Optional[typing.Type[tkmilan_var.Variable]] = None
 
-    def init_single(self, variable: var.Variable = None, gkeys: typing.Iterable[str] = None) -> None:
-        '''Setup all single widget stuff.'''
+    def init_single(self,
+                    variable: typing.Optional[tkmilan_var.Variable] = None,
+                    gkeys: typing.Optional[typing.Iterable[str]] = None,
+                    ) -> None:
+        '''Setup all single widget stuff.
+
+        Includes:
+        - Variable settings
+        - `tkmilan.mixin.MixinState.isNoneable` calculation
+        '''
         MixinWidget.__init__(self, gkeys=gkeys)
         self.variable = self.setup_variable(variable)
         MixinTraces.init_traces(self)
         if self.isNoneable is None:
             # Calculate isNoneable option
-            self.isNoneable = self.state_type is var.nothing
+            self.isNoneable = self.state_type is tkmilan_var.nothing
 
-    def setup_variable(self, variable: typing.Optional[var.Variable]) -> var.Variable:
+    def setup_variable(self, variable: typing.Optional[tkmilan_var.Variable]) -> tkmilan_var.Variable:
         assert self.state_type is not None
         if variable is None:
             variable = self.state_type()
@@ -508,14 +599,61 @@ class SingleWidget(MixinWidget, MixinStateSingle, MixinTraces):
     #    ...  # To Be overriden
 
 
+class ProxyWidget(SingleWidget):
+    '''Parent class of all proxy widgets. Special case of `SingleWidget`.
+
+    This is implemented as a class initializer that sets up the `wproxy
+    <MixinWidget.wproxy>`/`proxee <MixinWidget.proxee>` references, and returns
+    the child instance.
+
+    Note that creating an instance of this type will return the child widget
+    instance, not the proxy object. The rest of the library is aware of this.
+    The `ProxyWidget` object is available on the `wproxy <MixinWidget.wproxy>`
+    value.
+
+    See Also:
+        Check the Python documentation for the difference between
+        `object.__new__` and `object.__init__`.
+    '''
+    def __new__(cls: typing.Type[ProxyWidgetT], *args, **kwargs) -> ProxyWidgetT:
+        assert issubclass(cls, ProxyWidget)
+        proxy = super(ProxyWidget, cls).__new__(cls)
+        # Manually call the __init__ method (required since the class changes)
+        cls.__init__(proxy, *args, **kwargs)
+        proxee = proxy.proxee
+        assert proxee is not None
+        # Save a reference to the proxy object
+        proxee.wproxy = proxy
+        # Return a different type from `cls`:
+        return typing.cast(ProxyWidgetT, proxee)
+
+
 # TODO: Support MixinTraces? Synthetic trace of all sub widgets?
 class ContainerWidget(MixinWidget, MixinStateContainer):
     '''Parent class of all containers.'''
     layout: typing.Optional[str] = ''  # Automatic AUTO
+    layout_expand: bool = True
+    '''Should this container expand to fill the space on the parent widget.
 
-    def init_container(self, *args, layout: typing.Optional[str] = '', expand: bool = True, autogrow: bool = True, **kwargs):
+    Note this affects the **parent** grid, not the child grid on this container.
+    '''
+    layout_autogrow: bool = True
+    '''Should this container have its child columns and rows grow automatically.
+
+    This is equivalent to configuring the grid with the option ``weigth=1``.
+    '''
+
+    def init_container(self, *args,
+                       layout: typing.Optional[str] = '',
+                       **kwargs) -> None:
         '''
-        Setup all the container stuff
+        Setup all the container stuff.
+
+        Includes:
+        - Variable settings
+        - Sub-Widget settings
+        - Layout
+        - Defaults
         '''
         assert isinstance(self, (tk.Widget, tk.Tk)), f'{self} is not a valid tkinter.Widget'
         MixinWidget.__init__(self)
@@ -540,9 +678,10 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             # logger.debug('tk C: %r', self.children)
             widgets = {}
             dir_names = {id(getattr(self, name, None)): name for name in _new_names}
-            for widget in children:
-                assert isinstance(widget, MixinWidget), '{widget} is not a valid tkmilan widget'
-                if not widget.isHelper:
+            for widget_raw in children:
+                assert isinstance(widget_raw, MixinWidget), '{widget_raw} is not a valid tkmilan widget'
+                widget = widget_raw.proxee or widget_raw  # Save the child widget
+                if not (widget).isHelper:
                     wid = id(widget)
                     assert wid in dir_names, f'{self}: Missing "{widget}"[{widget!r}]'
                     name = dir_names[wid]
@@ -567,7 +706,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         else:
             # Fallback
             chosen_layout = autolayout.AUTO
-        self.layout_container(chosen_layout, expand, autogrow)
+        self.layout_container(chosen_layout)
         self.setup_defaults()
         self.after_idle(lambda: self.setup_adefaults())  # No need for a `TimeoutIdle` here
         assert hasattr(self, 'grid'), f'{self!r} should have a grid method'
@@ -582,24 +721,56 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         '''
         raise NotImplementedError
 
-    def var(self, cls: typing.Type[var.Variable], *, value=None, name=None) -> tk.Variable:
+    def var(self, cls: 'typing.Type[tkmilan_var.Variable]', *,
+            value=None,
+            name=None,
+            ) -> 'tkmilan_var.Variable':
         '''"Attach" a new variable to this container.
 
         Args:
             cls: The variable class.
 
-            value: The default value.
-                Optional, defaults to `None`.
-            name: The variable name.
-                Optional, defaults to an autogenerated name.
+            value: The default value. Optional, defaults to `None`.
+            name: The variable name. Optional, defaults to an autogenerated name.
 
         See Also:
-            gvar: Access the variable by name.
+            - `varSpecced`: Attach a specified variable to this container, with a name.
+            - `gvar`: Access the variable by name.
         '''
         vobj = cls(value=value)
-        assert isinstance(vobj, var.Variable), f'Class "{cls}" is not a "tk.Variable"'
+        assert isinstance(vobj, tkmilan_var.Variable), f'Class "{cls}" is not a "tk.Variable"'
         # Save the variables on the instance object
-        self._variables[name or str(vobj)] = vobj
+        vname = name or str(vobj)
+        self._variables[vname] = vobj
+        return vobj
+
+    def varSpecced(self, cls: 'typing.Type[tkmilan_var.VariableSpecced]', *,
+                   spec: spec.SpecCountable,
+                   value=None,
+                   name=None,
+                   ) -> 'tkmilan_var.VariableSpecced':
+        '''"Attach" a new specified variable to this container.
+
+        Args:
+            cls: The variable class.
+            spec: The variable specification.
+
+            value: Th default value. Optional, default to `None`.
+            name: The variable name. Optional, defaults to an autogenerated name.
+
+        See Also:
+            - `var`: Attach a non-speficied variable to this container, with a name.
+            - `gvar`: Access the variable by name.
+        '''
+        vobj = cls(value=value, spec=spec)
+        assert isinstance(vobj, tkmilan_var.Variable), f'Class "{cls}" is not a "tk.Variable"'
+        # Save the variables on the instance object
+        vname = name or str(vobj)
+        self._variables[vname] = vobj
+        # Set the default ASAP, on the root widget
+        # - This is to make sure this is only set once, even if used in several locations
+        # - Errors if this is called twice for the same name
+        self.wroot.tidle(vobj.setDefault, key=f'__:varSpecced_default:{vname}')
         return vobj
 
     def gvar(self, name: str):
@@ -611,13 +782,14 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             name: The variable name to search for.
 
         See Also:
-            var: Attach a variable to this container, with a name.
+            - `var`: Attach a non-speficied variable to this container, with a name.
+            - `varSpecced`: Attach a specified variable to this container, with a name.
         '''
         return self._variables[name]
 
-    def layout_container(self, layout: typing.Optional[str], expand: bool, autogrow: bool):
+    def layout_container(self, layout: typing.Optional[str]):
         assert isinstance(self, (tk.Widget, tk.Tk)), f'{self} is not a valid tkinter.Widget'
-        if expand:
+        if self.layout_expand:
             assert isinstance(self, tk.ttk.Widget), f'{self} is not a valid tkinter.ttk.Widget'
             self.grid(sticky=tk.NSEW)
         # Automatic Layout
@@ -626,19 +798,19 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             # if __debug__:
             #     logger.debug(f'{self}: => {len(self.widgets)} widgets')
             for idx, (arg, widget) in enumerate(zip(args, self.widgets.values())):
-                assert isinstance(widget, tk.Widget)
-                widget.grid(**arg.dict())
-        # TODO: Return a boolean/dataclass to control autogrowth
-        self.setup_layout(layout)
+                widget_real = widget.wproxy or widget
+                assert isinstance(widget_real, tk.Widget)
+                widget_real.grid(**arg.dict())  # Change the grid on the proxy widget
         self.layout = layout  # Setup the final layout setting
-        if autogrow:
+        if self.layout_autogrow:
             if size := self.gsize:
                 fn.configure_grid(self, [1] * size.columns, [1] * size.rows)
+        self.setup_layout(layout)  # Custom adjustments, after all automatic changes
 
     @property
     def gsize(self) -> model.GridSize:
         '''GUI grid size (according to the current child widgets).'''
-        return fn.grid_size(*self.widgets.values())
+        return fn.grid_size(*[w.wproxy or w for w in self.widgets.values()])  # Use the proxy widget
 
     def state_c(self, *, vid_upstream: typing.Optional[typing.Set[str]] = None) -> ContainerState:
         swidgets = {}
@@ -674,22 +846,22 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         return ContainerState(swidgets, cwidgets, self._variables, wvariables, dict(vwidgets),
                               vid_upstream=vid_upstream)
 
-    def setup_state(self, **kwargs):
+    def setup_state(self, **kwargs) -> typing.Mapping[str, model.WidgetDynamicState]:
         # Default State:
         # - All the attached variables
         # - All the shared variables
         # - All the single-variable widgets
         # - The container widgets, taking the existing variables into account
         container_state = self.state_c(**kwargs)
-        rvalue: typing.Mapping[str, model.WidgetDynamicState] = {}
-        wids_done = []
-        for vn, v in container_state.variables.items():
-            rvalue[vn] = model.WidgetDynamicState(v.get, v.set, noneable=False)
-        for v, ws in container_state.vwidgets.items():
-            if v is not None and len(ws) > 1:
-                wv = container_state.wvariables[v]
-                assert v not in rvalue, f'{self!r}: Aliased vwidgets "{v}"'
-                rvalue[v] = model.WidgetDynamicState(wv.get, wv.set, noneable=False)
+        rvalue: typing.MutableMapping[str, model.WidgetDynamicState] = {}
+        wids_done: typing.MutableSequence[str] = []
+        for vn, vobj in container_state.variables.items():
+            rvalue[vn] = model.WidgetDynamicState(vobj.get, vobj.set, noneable=False)
+        for vname, ws in container_state.vwidgets.items():
+            if vname is not None and len(ws) > 1:
+                wv = container_state.wvariables[vname]
+                assert vname not in rvalue, f'{self!r}: Aliased vwidgets "{vname}"'
+                rvalue[vname] = model.WidgetDynamicState(wv.get, wv.set, noneable=False)
                 wids_done.extend(ws)
         for n, w in container_state.swidgets.items():
             if n not in wids_done:
@@ -697,7 +869,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
                 rvalue[n] = model.WidgetDynamicState(
                     w.wstate_get,
                     w.wstate_set,
-                    noneable=w.isNoneable,
+                    noneable=w.isNoneable is True,
                 )
         vid_upstream = container_state.vid_upstream
         for n, wc in container_state.cwidgets.items():
@@ -705,7 +877,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             rvalue[n] = model.WidgetDynamicState(
                 partial(wc.state_get, vid_upstream=vid_upstream),
                 partial(wc.state_set, vid_upstream=vid_upstream),
-                noneable=wc.isNoneable,
+                noneable=wc.isNoneable is True,
                 container=True,  # Propagate container data
             )
         return rvalue
@@ -713,6 +885,8 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
     def setup_layout(self, layout: typing.Optional[str]) -> None:
         '''Useful for manual adjustments to the automatic layout.
 
+        This runs after all automatic layout settings are configured.
+
         Args:
             layout: This is the string passed to the upstream function.
 
@@ -721,14 +895,46 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         '''
         pass
 
-    def set_gui_substate(self, state: typing.Optional[model.GuiState] = None, **kwargs) -> None:
+    def set_gui_state(self, state: typing.Optional[model.GuiState] = None, _sub: bool = True, **kwargs) -> model.GuiState:
+        '''Set GUI State for itself, and optionally, for all sub-widgets.
+
+        .. warning:: Don't use this directly, unless you **really** know what you are doing.
+
+        Args:
+            _sub: Automatically run `set_gui_substate` with the same
+                `model.GuiState` object. Defaults to `True`.
+                Useful only for implementing special containers.
+
+        See Also:
+            `MixinWidget.gstate`: Property changed for all sub-widgets.
+        '''
+        self_state = super().set_gui_state(state, **kwargs)
+        if _sub:
+            self.set_gui_substate(self_state)
+        return self_state
+
+    def set_gui_substate(self, state: model.GuiState) -> None:
         '''Set GUI State for all sub-widgets.
+
+        .. warning:: Don't use this directly, unless you **really** know what you are doing.
+
+
+        .. note::
+
+            To control the GUI subwidget handling, this function can be
+            redefined (using extra care), using something like this:
+
+            .. code:: python
+
+                def set_gui_substate(self, state: tkmilan.model.GuiState):
+                    if self.some_condition is True:
+                        # Manipulate the `model.GuiState` object
+                        state.enabled = False
+                    super().set_gui_substate(state)
 
         See Also:
             `MixinWidget.gstate`: Property changed for all sub-widgets.
         '''
-        if state is None:
-            state = model.GuiState(**kwargs)
         for _, subwidget in self.widgets.items():
             subwidget.gstate = state
 
@@ -738,13 +944,15 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         Note this runs before the parent widget is complete ready.
 
         Useful to set default values.
+        Do not configure layout-related settings here, see `setup_layout
+        <ContainerWidget.setup_layout>`.
 
         Note:
             Available for subclass redefinition.
 
         See Also:
-            `setup_adefaults`: Run code after all widgets are stable (including
-            parent widgets in the tree).
+            `setup_adefaults <ContainerWidget.setup_adefaults>`: Run code after
+            all widgets are stable (including parent widgets in the tree).
         '''
         pass
 
@@ -757,7 +965,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             Available for subclass redefinition.
 
         See Also:
-            `setup_defaults`: Run code right after this widget is setup, before
+            `setup_defaults <ContainerWidget.setup_defaults>`: Run code right after this widget is setup, before
             all widgets are stable.
         '''
         pass
