@@ -9,6 +9,7 @@ import enum
 import abc
 from functools import cached_property
 import math
+import time
 from pathlib import Path
 from fractions import Fraction
 import tkinter as tk
@@ -30,6 +31,7 @@ TraceModeT = typing.Literal['read', 'write', 'unset']
 
 There is no Python documentation, see ``Tk`` `trace variable <https://www.tcl.tk/man/tcl/TclCmd/trace.html#M14>`_ documentation.
 '''
+compoundT = typing.Union['CP', bool, None]
 
 
 # Image Types
@@ -136,7 +138,7 @@ class CP(enum.Enum):
     '''OS-specific cardinal point.'''
 
 
-CP_Compound: typing.Mapping[typing.Union[CP, bool, None], str] = {
+CP_Compound: typing.Mapping[compoundT, str] = {
     CP.default: 'none',  # `image` if present, otherwise `text`
     CP.N: tk.BOTTOM,
     CP.S: tk.TOP,
@@ -151,10 +153,51 @@ CP_Compound: typing.Mapping[typing.Union[CP, bool, None], str] = {
 
 This is the mapping between `CP` and the ``compound`` configuration string used in several locations.
 
+Besides the cardinal points `CP.N`, `CP.S`, `CP.E`, `CP.W`, there are also special cases:
+
+- `CP.default`: Show only the image if given, otherwise show only label.
+- `True`: Show only the image
+- `False`: Show only the label
+- `None`: Show image and label, both centered
+
 There is no Python documentation, see ``Tk`` `compound <https://www.tcl.tk/man/tcl/TkCmd/ttk_widget.html#M-compound>`_ documentation.
 '''
 
 
+class Justification(enum.Enum):
+    '''A text justification anchor.
+
+    Corresponds to neatly to the same `tkinter` values, but it's simpler to
+    validate as an `enum`.
+
+    Note:
+        Not all widgets support ``NoJustify``, sometimes it's necessary to not
+        pass the ``justify`` option at all.
+    '''
+    NoJustify = None
+    '''OS-specific justification.'''
+    Left = tk.LEFT
+    '''Justify Left'''
+    Center = tk.CENTER
+    '''Justify Center'''
+    Right = tk.RIGHT
+    '''Justify Right'''
+
+
+Justification_CP: typing.Mapping[Justification, CP] = {
+    Justification.NoJustify: CP.default,
+    Justification.Left: CP.W,
+    Justification.Center: CP.center,
+    Justification.Right: CP.E,
+}
+'''Conversion between `Justification` and `CP` objects.
+
+This is useful to convert between types, since some widgets have weird
+interactions between ``justify`` and ``anchor`` settings.
+'''
+assert set(Justification_CP.keys()) == set(Justification), 'Justification_CP does not cover all cases'
+
+
 @dataclass
 class PixelSize:
     width: int
@@ -274,17 +317,19 @@ class Direction(enum.Enum):
     +-----+-----+-----+
     |*Cardinal Point* |
     +-----+-----+-----+
-    |     |  N  |     |
+    |     |  N  |  H  |
     +-----+-----+-----+
     |  W  |     |  E  |
     +-----+-----+-----+
-    |     |  S  |     |
+    |  V  |  S  |     |
     +-----+-----+-----+
 
     - **N**: North
     - **S**: South
     - **E**: East
     - **W**: West
+    - **H**: Reversed Horizontal, combines North and East
+    - **V**: Reversed Vertical, combines South and West
 
     '''
     # deltaRow, deltaColumn
@@ -292,6 +337,8 @@ class Direction(enum.Enum):
     S = (+1, 0)
     E = (0, +1)
     W = (0, -1)
+    H = (-1, +1)
+    V = (+1, -1)
 
     def __init__(self, dR: int, dC: int):
         # This is not really used anywhere, for now
@@ -331,9 +378,9 @@ class Direction(enum.Enum):
         if __debug__:
             logger_layout.debug('Grid[%s]: %d (%dx%d=%d) [D%d]', self.name, amount, rows, cols, size, extra_size)
         for idx in range(amount):
-            if self in (Direction.E, Direction.W):  # Horizontal
+            if self in (Direction.E, Direction.W, Direction.H):  # Horizontal
                 c = (idx // cols, idx % cols)
-            elif self in (Direction.N, Direction.S):  # Vertical
+            elif self in (Direction.N, Direction.S, Direction.V):  # Vertical
                 c = (idx % rows, idx // rows)
             else:
                 # Not generic for any delta...
@@ -345,9 +392,9 @@ class Direction(enum.Enum):
                 if self == Direction.N:
                     crow = crow - extra_size
                     crspan = 1 + extra_size
-                elif self == Direction.S:
+                elif self in (Direction.S, Direction.V):
                     crspan = 1 + extra_size
-                elif self == Direction.E:
+                elif self in (Direction.E, Direction.H):
                     ccspan = 1 + extra_size
                 elif self == Direction.W:
                     ccol = ccol - extra_size
@@ -412,20 +459,26 @@ class Direction(enum.Enum):
             for other in range(0, osize, this_width):
                 if __debug__:
                     logger_layout.debug('» T%d O%d TW%d | TA%d OS%d', idx, other, this_width, this_amount, osize)
-                if self in (Direction.E, Direction.W):  # Horizontal
+                if self in (Direction.E, Direction.W, Direction.H):  # Horizontal
                     # Grid: asize x osize
                     crow, ccol, crspan, ccspan = idx, other, 1, this_width
+                    if __debug__:
+                        logger_layout.debug('»»» %d+%dx%d+%d', crow, crspan, ccol, ccspan)
+                    if self.dR < 0:
+                        crow = asize - crow - crspan
                     if self.dC < 0:
                         ccol = osize - ccol - ccspan
-                    assert 0 <= crow <= asize and 0 <= crow + crspan - 1 <= asize
-                    assert 0 <= ccol <= osize and 0 <= ccol + ccspan - 1 <= osize
-                elif self in (Direction.N, Direction.S):  # Vertical
+                    assert 0 <= crow <= asize and 0 <= crow + crspan - 1 <= asize, f'R ({crow}+{crspan}/{asize})'
+                    assert 0 <= ccol <= osize and 0 <= ccol + ccspan - 1 <= osize, f'C ({ccol}+{ccspan}/{osize})'
+                elif self in (Direction.N, Direction.S, Direction.V):  # Vertical
                     # Grid: osize x asize
                     crow, ccol, crspan, ccspan = other, idx, this_width, 1
+                    if self.dC < 0:
+                        ccol = asize - ccol - ccspan
                     if self.dR < 0:
                         crow = osize - crow - crspan
-                    assert 0 <= crow <= osize and 0 <= crow + crspan - 1 <= osize
-                    assert 0 <= ccol <= asize and 0 <= ccol + ccspan - 1 <= asize
+                    assert 0 <= crow <= osize and 0 <= crow + crspan - 1 <= osize, f'R ({crow}+{crspan}/{osize})'
+                    assert 0 <= ccol <= asize and 0 <= ccol + ccspan - 1 <= asize, f'C ({ccol}+{ccspan}/{asize})'
                 if __debug__:
                     logger_layout.debug('»» %d %d | %d+%dx%d+%d', idx, other, crow, crspan, ccol, ccspan)
                 yield GridCoordinates(crow, ccol,
@@ -441,7 +494,7 @@ class WidgetDynamicState:
     Args:
         getter: Function to retrieve the state
         setter: Function to change the state
-        noneable: Is this widget noneable? See `mixin.MixinState.isNoneable`.
+        noneable: Is this widget noneable? See `tkmilan.mixin.MixinState.isNoneable`.
         container: Is this widget a container? Defaults to `False`.
     '''
     getter: typing.Callable
@@ -500,6 +553,35 @@ class WState(typing.Generic[wsT, wsST]):
     '''Get this information as a dictionary.'''
 
 
+@dataclass
+class WindowState:
+    '''GUI State for entire windows.
+
+    This object tracks changes to attributes that apply entire windows, not
+    individual widgets.
+
+    Args:
+        fullscreen: Show the window contents on the entire screen.
+
+    See Also:
+        This is used in `RootWindow`, as ``rgstate``.
+    '''
+
+    fullscreen: typing.Optional[bool]
+
+    def items(self):
+        '''Workaround for `dataclasses.asdict` issue.
+
+        This is a problem-free version of:
+
+        .. code:: python
+
+            return dataclasses.asdict(self).items()
+        '''
+        for f in fields(self):
+            yield (f.name, getattr(self, f.name))
+
+
 @dataclass
 class GuiState:
     '''Widget GUI state.
@@ -511,6 +593,7 @@ class GuiState:
     enabled: typing.Optional[bool] = None
     valid: typing.Optional[bool] = None
     readonly: typing.Optional[bool] = None
+    alternate: typing.Optional[bool] = None
 
     def items(self):
         '''Workaround for `dataclasses.asdict` issue.
@@ -577,6 +660,7 @@ GUI_STATES: typing.Mapping[str, GuiState_Tk] = {
     'enabled': GuiState_Tk(tk.DISABLED, invert=True),
     'valid': GuiState_Tk('invalid', invert=True),
     'readonly': GuiState_Tk('readonly', invert=False),
+    'alternate': GuiState_Tk('alternate', invert=False),
 }
 '''`GuiState` ``Tk`` metadata.
 
@@ -588,6 +672,7 @@ GUI_STATES_common = ('enabled', 'valid')
 See `GUI_STATES`.'''
 assert set(GUI_STATES.keys()) == set(f.name for f in fields(GuiState))
 assert set(GUI_STATES_common) < set(f.name for f in fields(GuiState))
+assert not (set(GUI_STATES.keys()) < set(('_internal', '_sub')))
 
 
 class MixinBinding:
@@ -1037,6 +1122,208 @@ class TimeoutIdle:
             return True
 
 
+class Interval:
+    '''Repeat an action with an interval between calls.
+
+    Can be scheduled and unscheduled separately from creation, see the
+    ``immediate`` argument.
+
+    Uses a `Timeout` internally.
+
+    Args:
+        widget: The widget to apply the interval to. When in doubt, use the
+            `wroot <mixin.MixinWidget.wroot>` widget.
+        action: The function to call when the interval elapses.
+        interval: The interval time, in milliseconds.
+            This only starts to count from the scheduling time, not the object
+            creation, but see ``immediate``.
+        immediate: Schedule interval on creation.
+            Defaults to `False`, set to `True` to schedule automatically.
+
+    See Also:
+        To run the action only once, see `Timeout`.
+    '''
+
+    scheduled: bool
+    '''Scheduled State.
+
+    Checks if the interval is scheduled to repeat.
+
+    The state is one of the following:
+
+    - `True`: Scheduled, the action will be repeated forever.
+    - `False`: Unscheduled, but the action might be repeated once again at
+      most.
+    '''
+
+    def __init__(self,
+                 widget: 'mixin.MixinWidget',
+                 action: typing.Callable,
+                 interval: int,
+                 immediate: bool = False,
+                 ):
+        self.scheduled = False
+        self.action = action
+        self.timeout = Timeout(widget, self._intervalElapsed, timeout=interval,
+                               immediate=False)
+        if immediate:
+            self.schedule()
+
+    def _intervalElapsed(self) -> None:  # Internal
+        '''Function to attach to the `Timeout` action.
+
+        The countdown for expiration of ``interval`` milliseconds starts when
+        this function is called.
+        '''
+        # Run the action
+        self.action()  # TODO: Suport `async`: Use `self.after(0, self.action)`, needs a lock
+        # `self.reschedule`
+        if self.scheduled:
+            self.timeout.reschedule()
+
+    def schedule(self, _e: typing.Any = None) -> None:
+        '''Schedule the interval to start ticking.
+
+        The countdown for expiration of ``interval`` milliseconds starts when
+        this function is called.
+
+        Make sure the interval is not scheduled already. When in doubt, use
+        `reschedule` instead.
+
+        Args:
+           _e: Unused, included for API compatibility with ``Tk`` events
+        '''
+        assert not self.scheduled, f'{self!r}: Already scheduled'
+        self.scheduled = True
+        self.timeout.reschedule()
+
+    def unschedule(self, _e: typing.Any = None, *, force: bool = False) -> None:
+        '''Cancel the interval.
+
+        This cancels the interval repetion, but after calling this function,
+        the action might still run, at most once. See ``force`` to control
+        this behaviour.
+
+        Args:
+            force: Unschedule on-flight interval, if exists.
+                Disabled by default.
+
+            _e: Unused, included for API compatibility with ``Tk`` events
+        '''
+        if __debug__:
+            if not self.scheduled:
+                logger.debug('%r: Already unscheduled', self)
+        self.scheduled = False
+        if force:
+            self.timeout.unschedule()
+
+    def reschedule(self, _e: typing.Any = None) -> None:
+        '''Reschedule the interval.
+
+        If the interval is already scheduled, this resets the interval, so it
+        starts ticking from this point. If the interval is unscheduled, it
+        schedules it.
+
+        Either way, after ``interval`` milliseconds of calling this function,
+        the action will run.
+
+        Args:
+            _e: Unused, included for API compatibility with ``Tk`` events
+        '''
+        if self.scheduled:
+            self.timeout.reschedule()
+        else:
+            self.schedule()
+
+
+class RateLimiter:
+    '''Rate Limit an action.
+
+    Define an action and a time limit. No matter how often you `hit` this
+    object, the time between actions is never less than the ``limit`` (in
+    milliseconds).
+
+    It's ready to use automatically, there's no need to "schedule" it, like
+    `Timeout`.
+    Uses a `Timeout` internally.
+
+    Args:
+        widget: The widget to apply the interval to. When in doubt, use the
+            `wroot <mixin.MixinWidget.wroot>` widget.
+        action: The function to rate limit.
+        limit: The rate-limit, in milliseconds. The ``action`` function is
+            called at most once every this time.
+
+    See Also:
+        See also other time-related classes like `Interval` and `Timeout`.
+    '''
+
+    _last_ns: int
+    '''NanoSeconds since the last action execution.
+
+    On init, a negative value is stored
+    '''
+
+    def __init__(self, widget: 'mixin.MixinWidget', action: typing.Callable, limit: int):
+        self.limit_ns = limit * int(1e6)  # ms -> ns
+        self.action = action
+        self.timeout = Timeout(widget, self._doAction, timeout=limit,
+                               immediate=False)
+
+        self._last_ns = - self.limit_ns - 1  # Guarantee the first time will work
+
+    def _doAction(self) -> None:  # Internal
+        '''Function to attach to the `Timeout` action.
+
+        The last action execution time is stored here.
+        '''
+        # Re-Arm first (avoid race condition)
+        self.timeout.unschedule()
+        # Run the action
+        self.action()
+        # Mark the last action time
+        self._last_ns = time.monotonic_ns()
+
+    def hit(self, _e: typing.Any = None) -> bool:
+        '''Mark the action as "active".
+
+        When this function is called, the action may or may not run, depending
+        on the last time it ran. If the rate limit is respected, the action
+        runs, otherwise it will wait at least the ``limit`` time. Calling this
+        function again before the limit is elapsed will reset the timer.
+
+        Args:
+           _e: Unused, included for API compatibility with ``Tk`` events
+
+        Returns
+        Returns:
+            If the action ran this time, return `True`, otherwise return `False`.
+        '''
+        monotime = time.monotonic_ns()
+        if monotime - self._last_ns > self.limit_ns:
+            self._doAction()  # TODO: Suport `async`: Use `self.after(0, self.action)`, needs a lock
+            return True
+        else:
+            # logger.debug('Rate Limited. Elapsed %d ns', monotime - self._last_ns)
+            self.timeout.reschedule()
+            # TODO: Return the elapsed time?
+            return False
+
+    def cancel(self, _e: typing.Any = None) -> None:
+        '''Cancel the next action execution.
+
+        This cancels an action execution, when one is pending.
+
+        Be advised, this means a triggered action **WILL NOT RUN**. When used
+        for event processing, means events will be lost. Make sure you know
+        what you are doing.
+
+        Args:
+            _e: Unused, included for API compatibility with ``Tk`` events
+        '''
+        self.timeout.unschedule()
+
+
 @dataclass
 class NotebookTab:
     '''Information for each `Notebook <tkmilan.Notebook>` tab.
